import torch
import torch.nn as nn

import numpy as np      # Fixme: +

from ..registry import LOSSES
from .utils import weighted_loss


@weighted_loss
def smooth_l1_loss(pred, target, beta=1.0):
    assert beta > 0
    # print("pred",pred)
    # print("target",target)
    # print("pred",pred.size())
    # print("target", target.size())
    assert pred.size() == target.size()  # and \
    assert target.numel() > 0
    diff = torch.abs(pred - target)
    # diff = torch.where(diff<0.5, 0, diff)
    loss = torch.where(diff < beta, 0.5 * diff * diff / beta,
                       diff - 0.5 * beta)
    return loss


@LOSSES.register_module
class SmoothL1Loss(nn.Module):

    def __init__(self, beta=1.0, reduction='mean', loss_weight=1.0, pick=-1):
        super(SmoothL1Loss, self).__init__()
        self.beta = beta
        self.reduction = reduction
        self.loss_weight = loss_weight
        self.pick = pick

    def forward(self,
                pred,
                target,
                weight=None,
                avg_factor=None,
                reduction_override=None,
                **kwargs):
        assert reduction_override in (None, 'none', 'mean', 'sum')
        reduction = (
            reduction_override if reduction_override else self.reduction)
        if isinstance(self.pick, int):              # Fixme: +
            if self.pick != -1:
                if weight is not None:
                    weight = torch.reshape(weight, target.size())
                inds = np.where(target.cpu()==self.pick)
                target = target[inds]
                pred = pred[inds]
                weight = weight[inds]
        elif isinstance(self.pick, tuple):
            if weight is not None:
                weight = torch.reshape(weight, target.size())
            num_area = np.arange(self.pick[0], self.pick[1]+1)
            inds = np.isin(target.cpu(), num_area)
            target = target[inds]
            pred = pred[inds]
            weight = weight[inds]

        loss_bbox = self.loss_weight * smooth_l1_loss(
            pred,
            target,
            weight,
            beta=self.beta,
            reduction=reduction,
            avg_factor=avg_factor,
            **kwargs)
        return loss_bbox
